@@ -599,7 +599,9 @@ debug_logprintf (const char *fmt, ...)
       struct logvprintf_state lpstate;
       bool done;
 
+#ifndef TESTING
       check_redirect_output ();
+#endif
       if (inhibit_logging)
         return;
 
@@ -676,9 +678,16 @@ log_close (void)
 {
   int i;
 
-  if (logfp && (logfp != stderr))
-    fclose (logfp);
+  if (logfp && logfp != stderr && logfp != stdout)
+    {
+      if (logfp == stdlogfp)
+        stdlogfp = NULL;
+      if (logfp == filelogfp)
+        filelogfp = NULL;
+      fclose (logfp);
+    }
   logfp = NULL;
+
   inhibit_logging = true;
   save_context_p = false;
 
@@ -965,7 +974,7 @@ check_redirect_output (void)
     {
       pid_t foreground_pgrp = tcgetpgrp (STDIN_FILENO);
 
-      if (foreground_pgrp != -1 && foreground_pgrp != getpgrp ())
+      if (foreground_pgrp != -1 && foreground_pgrp != getpgrp () && !opt.quiet)
         {
           /* Process backgrounded */
           redirect_output (true,NULL);
